# basin_goldberg.py

# This is a demo code that attempts to reimplement
# the code from the C++ version of basin_goldberg 
# (found in basin_kernel/trunk/basin_local)

# Please note that there are a few lines below
# that will likely need to be changed to match
# the specific layout of files on your filesystem.
# These lines have been marked with ###CHANGE_NEEDED###

import ipython1.kernel.api as kernel

rc = kernel.RemoteController(('localhost',10105))

# This string contains the commands to be run on the cluster
# Standard Python formatting should be used within here
cmd = """
# starting here, commands are run on cluster

from random import random
import math
from basin import *

###CHANGE_NEEDED### These lines should point to zcatalog.dat and
# zmask.dat in the cluster's filesystem
catalog = Region("/home/user/data/zcatalog.dat")
mask    = Region("/home/user/data/zmask.dat")

ra  = catalog.get_attribute("ra")
dec = catalog.get_attribute("dec")
z   = catalog.get_attribute("z")

ra_mask  = mask.get_attribute("ra")
dec_mask = mask.get_attribute("dec")

nmask = ra_mask.get_dims()[0]

z_mask = Attribute(nmask,0.0)

zmax = 1.0

zval = [0 for x in range(0, nmask)]
for i in range(0, nmask):
    zval[i] = zmax * random()

### This is slow, but will have to do 
### until the put method is added
for i in range(0, nmask):
    z_mask[i] = zval[i]

#plot(z_mask,"z_mask")

(x1,y1,z1,xm,ym,zm) = [Attribute() for x in range(0, 6)]

lcdm = cosmology()

r = lcdm.redshift_to_distance(z,0)
plot(z,r,"comoving distance")

r_mask = lcdm.redshift_to_distance(z_mask)
ierr = equatorial_to_xyz(ra,dec,r,x1,y1,z1)
ierr = equatorial_to_xyz(ra_mask, dec_mask, r_mask, xm, ym, zm)

n_tmp = 100
nx = [n_tmp,n_tmp]

dens = compute_density(x1,y1,n_tmp)
plot(x1,y1)
plot(dens, "density")

dens_mask = compute_density(xm,ym,n_tmp)

plot(dens_mask,"dens_mask")

select = dens_mask/(dens_mask+0.0001)

plot(select,"select")

dens_normal = dens*select/(dens_mask+0.0001)

ave = mean(dens_normal,select)
sig = stdev(dens_normal, select)

print "Mean = ", ave
print "Stdev = ", sig

delta = (dens_normal/ave-1)*select

plot(delta,"delta")

hist = histogram(delta, -0.75, 0.5, 20)

plot_hist(hist, -0.75, 0.5)

nx_kernel = [5,5]

kernel = [0 for x in range(0,25)]

ktot = 0

for i in range(0, nx_kernel[0]):
    for j in range(0, nx_kernel[1]):
        kernel[i*nx_kernel[1]+j] = math.exp(-((i-2)*(i-2)+(j-2)*(j-2)))
        ktot = ktot + math.exp(-((i-2)*(i-2)+(j-2)*(j-2)))

for i in range(0,25):
    kernel[i] = kernel[i] / ktot

delta_smooth = convolve(delta,kernel,nx_kernel)*select
print "Convolution done"

plot(delta_smooth, "delta_smooth")

radec = compute_density(ra,dec,20)

radec_k = fft(radec)

radec_realk = real(radec_k)
radec_imagk = imag(radec_k)

plot(radec_realk, "real")
plot(radec_imagk, "imag")

pk = power_spectrum(radec)
plot(pk,"power spectrum")

"""

# Send our commands to the server
rc.executeAll(cmd)
