#!/usr/bin/env python

import os
import sys
import time

# Modules for uid/gid manipulation
import pwd
import grp

def home ():
	return str (os.environ.get ("HOME"))

def lsDir  (path = home (), showHidden = False):
	if not os.path.exists (path):
		return
	dirs = os.listdir (path)
	ret = "\n"
	for dir in dirs:
		if os.path.isdir (str (path) + "/" + str (dir)) and (showHidden or dir[0] != "."):
			stat = os.stat (path + "/" + dir)
			# ret += dirname$uid$gid$bytes$accesstime$modifytime$createtime
			ret +=  dir + "$" + getUser (stat[4]) + "$" + getGid (stat[5]) + "$" + getSize (stat[6]) \
				+ "$" + strTime (stat[7]) + "$" + strTime (stat[8]) + "$" + strTime (stat[9]) + "\n"
	return ret

def lsFile (path = home (), showHidden = False):
	if not os.path.exists (path):
		return
	files = os.listdir (path)
	ret = "\n"
	for file in files:
		if os.path.isfile (str (path) + "/" + str (file)) and (showHidden or file[0] != "."):
			stat = os.stat (path + "/" + file)
			# ret += filename$uid$gid$bytes$accesstime$modifytime$createtime
			ret +=  file + "$" + getUser (stat[4]) + "$" + getGid (stat[5]) + "$" + getSize (stat[6]) \
				+ "$" + strTime (stat[7]) + "$" + strTime (stat[8]) + "$" + strTime (stat[9]) + "\n"
	return ret

def strTime (val):
	_time = time.localtime (val)	
	ret = getDay (_time[6]) + " " + str (_time[2]) + ", " + getMonth (_time[1]) + " " + str (_time[0])
	return ret

def getDay (val, long = False):
	days = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
	if long:
		return days[val]
	return days[val][:3]

def getMonth (val, long = False):
	val -= 1
	months = ["January", "February", "March", "April", "May", "June", "July", "August", \
		"September", "October", "November", "December"]
	if long:
		return months[val]
	return months[val][:3]

def getSize (val):
	ranks = [" bytes", " KB", "MB", "GB", "TB"]
	rank = 0
	while val > 1024 and rank < len (ranks):
		val = round (val / 1024.0, 1)
		rank += 1
	return str (val) + ranks[rank]

# Find a way to convert a user-id to the user's name
def getUser (val):
	try:
	# Convert a uid to a name
		return str (pwd.getpwuid (val)[0])
	except:
	# In case the file has a no-longer existing uid
	# just return the uid
		return str (val)

# Find a way to convert a group-id to the group's name
def getGid (val):
	try:
	# Convert a gid to a name
		return str (grp.getgrgid (val)[0])
	except:
	# In case the file has a no-longer existing gid
	# just return the gid
		return str (val)
