#include "Attribute_wrapper.h"
#include "basin_wrapper.h"

//boost::python::object Attribute__to_list(Attribute &a) {
//    if(BASIN_ID == 0) {
//        vector<double> vec(a.size());
//
//        a.get(0, a.size()-1, &vec[0]);
//
//        boost::python::list l(vec);
//
//        return l;
//    }
//
//    return boost::python::object();
//}

#ifdef VECTOR23
typedef boost::variant< double, int, float, Vector3, Vector2, char, fftw_complex > bv1;
#else
typedef boost::variant< double, int, float, char, fftw_complex > bv1;
#endif

vector<bv1> Attribute__to_list(Attribute &a) 
{
    vector<bv1> r;

    if(BASIN_ID == 0) {
        r = a.get_data_elements(0, a.size()-1);
    }

    return r;
}

vector<bv1> Attribute__local_to_list(Attribute &a) 
{
    vector<bv1> r;

    int l, h;
    
    a.get_gen_distribution_index(BASIN_ID, &l, &h);

    r = a.get_data_elements(l, h);

    return r;
}


/* 
 * This is a multiple dimension lookup into the class, should support
 * a[1] a[1,2] and so on.
 *
 */
boost::python::object Attribute__getitem__(Attribute &a, boost::python::object key) 
{

    int key_len = 0;

    key_len = get_py_object_len(key);
    
    vector<int> c_args(key_len);

    // Here we will expload the args given to us by Python
    // then we will call the appropriate C++ function to 
    // ask for the element
    if(key_len > 0) { //The py object can be iterated over, so extract each item
        for(int i = 0; i < key_len; i++)
        {
            c_args[i] = boost::python::extract<int>(key[i]);
        }
    }
    else { //The python object is a scalar, so try to extract an int
        c_args.resize(1);
        c_args[0] = boost::python::extract<int>(key);
    }

    bv1 curr_elem = (a.get_element_variant(c_args)).value;

    boost::python::object py_obj = boost::apply_visitor(get_py_object(), curr_elem);
    return py_obj;
}

/*
 * Creates a linear buffer of type T from an iterable Python object (typically a list). 
 */
template<typename T>
T* extractBufferFromPyList(boost::python::object data, int size)
{
    T* buf = new T[size];
    for(int i = 0; i < size; i++)
    {
        buf[i] = boost::python::extract<T>(data[i]);
    }

    return buf;
}

/*
 * Wrapper for the Attribute class get and put method. This function accepts python objects and
 * based on the data type associated with the Attribute tries to extract data of the same 
 * type from the Python object `data` and place this data in the Attribute at the given
 * indexes.
 */
void Attribute__put__(Attribute &a, boost::python::object li, boost::python::object hi,
        boost::python::object data)
{
    Attribute_Data_Type T = a.get_data_type();
    
    int h = boost::python::extract<int>(hi);
    int l = boost::python::extract<int>(li);
    int size = h - l + 1;

    if(T == BASIN_DATA_DOUBLE) {
        double *buf = extractBufferFromPyList<double>(data, size);
        a.put(l, h, buf);
    }
    else if(T == BASIN_DATA_INT) {
        int *buf = extractBufferFromPyList<int>(data, size);
        a.put(l, h, buf);
    }
    else if(T == BASIN_DATA_FLOAT) {
        float *buf = extractBufferFromPyList<float>(data, size);
        a.put(l, h, buf);
    }
}

vector<bv1> Attribute__get__(Attribute &a, boost::python::object li, boost::python::object hi){
  vector<bv1> r;
  Attribute_Data_Type T = a.get_data_type();
  int h = boost::python::extract<int>(hi);
  int l = boost::python::extract<int>(li);
  r=a.get_data_elements(l,h);
  return r;
}

/**
 * This is a multiple dimension set method for Attributes.
 * It will support any dimensionality that the C++ Attribute class handles.
 *
 */
void Attribute__setitem__(Attribute &a, boost::python::object key, boost::python::object value) 
{
    
    int key_len = 0;

    key_len = get_py_object_len(key);

    vector<int> c_args(key_len);

    if(key_len > 0) {
        for(int i = 0; i < key_len; i++)
        {
            c_args[i] = boost::python::extract<int>(key[i]);
        }
    }
    else {
        c_args.resize(1);
        c_args[0] = boost::python::extract<int>(key);
    }

    // We need to see what type of object is actually in the Attribute.
    Attribute_Data_Type t = a.get_data_type();

    // We must check against all of the possible types in the boost variant
    // bv1.
    // If the definition of a bv1 ever changes, this list will need to
    // be updated.
    if(t == BASIN_DATA_DOUBLE) {
        double temp = boost::python::extract<double>(value);
        a.set_element(c_args, temp);
    }
    else if(t == BASIN_DATA_INT) {
        int temp = boost::python::extract<int>(value);
        a.set_element(c_args, temp);
    }
    else if(t == BASIN_DATA_FLOAT) {
        float temp = boost::python::extract<float>(value);
        a.set_element(c_args, temp);
    }

    // Attribute::set_element() doesn't appear to work with these types
    //else if(t == BASIN_DATA_VECTOR3)
    //else if(t == BASIN_DATA_VECTOR2)
    //else if(t == BASIN_DATA_CHAR))
    //else if(t == BASIN_DATA_COMPLEX)

}

Attribute Attribute__and__(Attribute &self, Attribute b) 
{
    return (self && b);
}

Attribute Attribute__or__(Attribute &self, Attribute b) 
{
    return (self || b);
}


Attribute (*where_ptr1)(const Attribute& a, const Attribute &h, const Attribute &j) = &where;

Attribute (*where_ptr2)(const Attribute& a, const double &h, const Attribute &j) = &where;

Attribute (*where_ptr3)(const Attribute& a, const float &h, const Attribute &j) = &where;

Attribute (*where_ptr4)(const Attribute& a, const int &h, const Attribute &j) = &where;

Attribute (*where_ptr5)(const Attribute& a, const fftw_complex &h, const Attribute &j) = &where;

#ifdef VECTOR23
Attribute (*where_ptr6)(const Attribute& a, const Vector3 &h, const Attribute &j) = &where;

Attribute (*where_ptr7)(const Attribute& a, const Vector2 &h, const Attribute &j) = &where;
#endif

Attribute (*where_ptr8)(const Attribute& a, const char &h, const Attribute &j) = &where;

Attribute (*where_ptr9)(const Attribute& a, const Attribute &j,const double &h) = &where;

Attribute (*where_ptr10)(const Attribute& a, const Attribute &j,const float &h) = &where;

Attribute (*where_ptr11)(const Attribute& a, const Attribute &j,const int &h) = &where;

Attribute (*where_ptr12)(const Attribute& a, const Attribute &j,const fftw_complex &h) = &where;

#ifdef VECTOR23
Attribute (*where_ptr13)(const Attribute& a, const Attribute &j,const Vector3 &h) = &where;

Attribute (*where_ptr14)(const Attribute& a, const Attribute &j,const Vector2 &h) = &where;
#endif

Attribute (*where_ptr15)(const Attribute& a, const Attribute &j,const char &h) = &where;

Attribute (*where_ptr16)(const Attribute& a, const int &j,const int &h) = &where;

Attribute (*where_ptr17)(const Attribute& a, const float &j,const float &h) = &where;

Attribute (*where_ptr18)(const Attribute& a, const double &j,const double &h) = &where;

Attribute (*where_ptr19)(const Attribute& a, Attribute &h)=&where;

Attribute (*attribute_index_ptr1)(Attribute&, int) = &attribute_index;
Attribute (*attribute_index_ptr2)(vector<int>, int, int) = &attribute_index;
Attribute (*attribute_index_ptr3)(int) = &attribute_index;
Attribute (*recenter_ptr)(Attribute&, vector<int>) = &recenter;
Attribute (*resort_ptr)(Attribute&, Attribute&) = &resort;
Attribute (*conj_ptr)(Attribute&) = &conj;
Attribute (*fix_ptr)(const Attribute& a) = &fix;
Attribute (*real_ptr) (const Attribute& a) = &real;
double     (*min_ptr)(Attribute&) = &min;
double     (*max_ptr)(Attribute&) = &max;
void       (Attribute::*set_context_ptr)(Context*) = &Attribute::set_context;



void export_Attribute() 
{
    /* Begin function pointers for Attribute Class*/
    class_<Attribute>("Attribute")
        .def(init< >())
        .def(init<Attribute>())
        .def(init<vector<int>, double>())
        .def(init<vector<int>, int>())
        .def(init<int, double>())
        .def(init<int, int>()) 
        .def(self + Attribute())
        .def(self + int())
        .def(int() + self)
        .def(self + float())
        .def(float() + self)
        .def(self - Attribute())
        .def(self - int())
        .def(int() - self)
        .def(self - float())
        .def(float() - self)
        .def(self * Attribute())
        .def(self * int())
        .def(int() * self)
        .def(self * float())
        .def(float() * self)
        .def(self / Attribute())
        .def(self / int())
        .def(int() / self)
        .def(self / float())
        .def(float() / self)
        .def(self < Attribute())
        .def(self < int())
        .def(self < double())
        .def(self > Attribute())
        .def(self > int())
        .def(self > float())
        .def(self >= Attribute())
        .def(self >= int())
        .def(self >= float())
        .def(self <= Attribute())
        .def(self <= int())
        .def(self <= float())
        .def(self == Attribute())
        .def(self == int())
        .def(self == float())
        .def(self != Attribute())
        .def(self != int())
        .def(self != float())
        // Note: These relation operators are '|' and '&' in Python
        // They don't have the same precedence as C++'s '||' and '&&'
        // operators, so this needs to be taken into consideration
        .def("__and__", &Attribute__and__)
        .def("__or__", &Attribute__or__)
        .def("set_element", &Attribute__setitem__)
        .def("__getitem__", &Attribute__getitem__)
        .def("__setitem__", &Attribute__setitem__)
        .def("size", &Attribute::size)
        .def("get_dims", &Attribute::get_dims)
        .def("fill", &Attribute::fill<double>)
        .def("fill", &Attribute::fill<int>)
        .def("get_name", &Attribute::get_name)
        .def("get_context", &Attribute::get_context_pointer, return_value_policy<reference_existing_object>())
        .def("to_list", &Attribute__to_list)
        .def("local_to_list", &Attribute__local_to_list)
        .def("set_context", set_context_ptr, (arg ("c")))
        .def("get_data_type", &Attribute::get_data_type)
        .def("put", &Attribute__put__)
      .def("get", &Attribute__get__)
        ;
    
    // where function is not a memeber of the Attribute class
    // but only works with Attributes, so it is defined here
    def("where", where_ptr1);
    def("where", where_ptr2);
    def("where", where_ptr3);
    def("where", where_ptr4);
    def("where", where_ptr5);
#ifdef VECTOR23
    def("where", where_ptr6);
    def("where", where_ptr7);
#endif
    def("where", where_ptr8);
    def("where", where_ptr9);
    def("where", where_ptr10);
    def("where", where_ptr11);
    def("where", where_ptr12);
#ifdef VECTOR23
    def("where", where_ptr13);
    def("where", where_ptr14);
#endif
    def("where", where_ptr15);
    def("where", where_ptr16);
    def("where", where_ptr17);
    def("where", where_ptr18);
    def("where", where_ptr19);

    def ("attribute_index", attribute_index_ptr1, (arg("a"), arg("dir")));
    def ("attribute_index", attribute_index_ptr2, (arg("nx"), arg("dir"), arg("order_type")=0));
    def ("attribute_index", attribute_index_ptr3, (arg("npts")));
    def ("recenter", recenter_ptr, (arg("attr"), arg("new_ll")));
    def ("resort", resort_ptr, (arg ("attr"), arg ("id")));
    def ("conj", conj_ptr, (arg("a")));
    def ("fix", fix_ptr, (arg("a")));
    def ("real", real_ptr,(arg("a")));
    def ("min", min_ptr, (arg("a")));
    def ("max", max_ptr, (arg("a")));
}

