#include "Cosmology_wrapper.h"
#include "basin_wrapper.h"



void export_Cosmology() {

    /* Function pointers needed for cosmology class*/
    Attribute& (cosmology::*cosmology_redshift_to_distance1)(Attribute& redshift, int distance_type, int hflag)=&cosmology::redshift_to_distance;
    Attribute* (cosmology::*cosmology_redshift_to_distance2)(List* listptr,string zname,string rname,
							     int distance_type,int hflag)=
      &cosmology::redshift_to_distance;
    Attribute (cosmology::*cosmology_redshift_to_growth1)(Attribute& redshift)=&cosmology::redshift_to_growth;
    Attribute* (cosmology::*cosmology_redshift_to_growth2)(List* listptr,string zname,string dname)=
      &cosmology::redshift_to_growth;
    double (cosmology::*cosmology_redshift_to_growth3)(double z)=&cosmology::redshift_to_growth;
    Attribute (cosmology::*cosmology_hubble1)(Attribute& a)=&cosmology::hubble;
    double (cosmology::*cosmology_hubble2)(double a)=&cosmology::hubble;
    double (cosmology::*powerspec1)(double k,double z)=&cosmology::power_spectrum;
    vector<double> (cosmology::*powerspec2)(vector<double> k, double z)=&cosmology::power_spectrum;
    Attribute& (cosmology::*powerspec3) (Attribute& k,double z)=&cosmology::power_spectrum;
    
    int (cosmology::*set_omega_m_ptr)(double val)=&cosmology::set_omega_m;
    int (cosmology::*set_omega_l_ptr)(double val)=&cosmology::set_omega_l;
    int (cosmology::*set_h_ptr)(double val)=&cosmology::set_h;
    int (cosmology::*set_sig_8_ptr)(double val)=&cosmology::set_sig_8;


    /* End pointers for cosmology class*/


    class_<cosmology>("cosmology")
      .def(init< >())
      .def("redshift_to_distance",cosmology_redshift_to_distance1,
	   (arg("redshift"),arg("distance_type")=0,arg("hflag")=0),return_value_policy<reference_existing_object>())
      .def("redshift_to_distance",cosmology_redshift_to_distance2,
	   (arg("list_ptr"),arg("zname")=0,arg("rname")="r",arg("distance_type")=0,
	    arg("hflag")=0),return_value_policy<reference_existing_object>())
      .def("redshift_to_growth",cosmology_redshift_to_growth1,
	   (arg("redshift")))
      .def("redshift_to_growth",cosmology_redshift_to_growth2,
	   (arg("listptr"),arg("zname")=0,arg("dname")="d"),return_value_policy<reference_existing_object>())
      .def("redshift_to_growth",cosmology_redshift_to_growth3,
	   (arg("z")))
      .def("hubble",cosmology_hubble1,(arg("a")))
      .def("hubble",cosmology_hubble2,(arg("a")))
      .def("power_spectrum",powerspec1,(arg("k"),arg("z")=0))
      .def("power_spectrum",powerspec2,(arg("k"),arg("z")=0))
      .def("power_spectrum",powerspec3,(arg("k"),arg("z")=0),return_value_policy<reference_existing_object>())
      .def("get_omega_m",&cosmology::get_omega_m)
      .def("get_omega_l",&cosmology::get_omega_l)
      .def("get_h",&cosmology::get_h)
      .def("get_sig_8",&cosmology::get_sig_8)
      .def("set_omega_m",set_omega_m_ptr,(arg("omega_m")))
      .def("set_omega_l",set_omega_l_ptr,(arg("omega_l")))
      .def("set_h",set_h_ptr,(arg("h")))
      .def("set_sig_8",set_sig_8_ptr,(arg("sig8")))
      ;
}
