#ifndef __LINEARALGEBRA_WRAPPER_H__
#define __LINEARALGEBRA_WRAPPER_H__

#include "basin_wrapper.h"

#include <Python.h>
#include <boost/python.hpp>

typedef matrix basin_matrix;

/*
 * Converts Basin's matrix class to a native python type.
 * This conversion relies on numpy.matrix being installed
 * on the system running python.
 */

struct basin_matrix_to_py_matrix
{
    static PyObject* convert(matrix m) {

        PyObject *numpy_obj, *numpy_matrix;

        numpy_obj = PyImport_ImportModule("numpy");
        
        if(numpy_obj) {
            numpy_matrix = PyObject_GetAttrString(numpy_obj, "matrix");
            
            if(!numpy_matrix) {
                cerr << "Error: Numpy Matrix type not found!\n";
                
                Py_INCREF(Py_None);
                return Py_None;
            }
        }
        else {
            cerr << "Error: Numpy module not found!\n";

            Py_INCREF(Py_None);
            return Py_None;
        }


        const double* data = m.get_data();
        int rows = m.get_n();
        int cols = m.get_m();

        boost::python::list mlist; //temp list to be passed to matrix constructor
        
        // copy the data from the basin data structure
        // to a 2d python list
        for(int i = 0; i < rows; i++) {
            boost::python::list t;
            
            for(int j = 0; j < cols; j++) {
                t.append(*(data+(i*cols+j)));
            }
            
            mlist.append(t);
        }
        
        PyObject *m_args = PyTuple_Pack(1, mlist.ptr());
        PyObject *n_matrix = PyObject_CallObject(numpy_matrix, m_args);

        // Cleanup
        Py_DECREF(m_args);
        Py_DECREF(numpy_obj);
        Py_DECREF(numpy_matrix);

        
        return n_matrix;
    }
};



#endif


