#include "StellarDynamics_wrapper.h"
#include "basin_wrapper.h"


void export_StellarDynamics() {


    /* Function pointers for Stellar Dynamics package*/
    double (*compute_local_density_ptr1)(vector<double> &x, vector<double> &y, vector<double> &z, vector<double> &m, const int currPart, const int k) = &compute_local_density;
    double (*compute_local_density_ptr2)(double x[], double y[], double z[], double m[], const int currPart, const int npart, const int k, double MAX_DIST) = &compute_local_density;
    vector<vector<double> > (*radial_profile_ptr1)(Attribute &x, Attribute &y, Attribute &z, Attribute &A, int nbins, double rmin, double rmax, string scale) = &radial_profile;
    vector<vector<double> > (*radial_profile_ptr2)(Attribute &x, Attribute &y, Attribute &z, Attribute &A, Attribute &filter, int nbins, double rmin, double rmax, vector<double> filter_range, string scale) = &radial_profile;
    vector<vector<double> > (*density_profile_ptr1)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, int nbins, double rmin, double rmax, string compute, string scale) = &density_profile;
    vector<vector<double> > (*density_profile_ptr2)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, Attribute &filter, int nbins, double rmin, double rmax, vector<double> filter_range, string compute, string scale) = &density_profile;
    vector<vector<double> > (*disp_profile_ptr1)(Attribute &x, Attribute &y, Attribute &z, Attribute &A, int nbins, double rmin, double rmax, string scale) = &disp_profile;
    vector<vector<double> > (*disp_profile_ptr2)(Attribute &x, Attribute &y, Attribute &z, Attribute &A, Attribute &filter, int nbins, double rmin, double rmax, vector<double> filter_range, string scale) = &disp_profile;
    vector<double> (*lag_rad_ptr1)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, vector<double> radii) = &lag_rad;
    vector<vector<double> > (*lag_rad_ptr2)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, Attribute &filter, vector<double> radii, vector<double> filter_range)= &lag_rad;
    vector<vector<double> > (*shape_profile_ptr1)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, int nbins, double rmin, double rmax, string scale) = &shape_profile;
    vector<vector<double> > (*shape_profile_ptr2)(Attribute &x, Attribute &y, Attribute &z, Attribute &mass, Attribute &filter, int nbins, double rmin, double rmax, vector<double> filter_range, string scale) = &shape_profile;
    /* End function pointers for Stellar Dynamics*/

    /* Begin wrapping of Stellar Dynamics package */
    def("center_of_mass", &center_of_mass);
    def("center_of_mass_2d", &center_of_mass_2d);
    def("compare_vector", &compare_vector);
    def("compute_local_density", compute_local_density_ptr1);
    def("compute_local_density", compute_local_density_ptr2, (arg("x"), arg("y"), arg("z"), arg("m"), arg("currPart"), arg("npart"), arg("k"), arg("MAX_DIST")=1.e10));
    def("density_list", &density_list, (arg("x"),arg("y"),arg("z"),arg("mass"),arg("neighbors")=6));
    def("density_list_vector", &density_list_vector,(arg("x"),arg("y"),arg("z"),arg("mass"),arg("neighbors")=6));
    def("density_center", &density_center);
    def("density_center_maxpos", &density_center_maxpos);
    def("core_radius", &core_radius);
    def("radial_profile", radial_profile_ptr1, (arg("x"), arg("y"), arg("z"), arg("A"), arg("nbins"), arg("rmin"), arg("rmax"), arg("scale")="linear"));
    def("radial_profile", radial_profile_ptr2, (arg("x"), arg("y"), arg("z"), arg("A"), arg("filter"), arg("nbins"), arg("rmin"), arg("rmax"), arg("filter_range"), arg("scale")="linear"));
    def("density_profile", density_profile_ptr1,(arg("x"), arg("y"), arg("z"), arg("mass"), arg("nbins"), arg("rmin"), arg("rmax"), arg("compute")="mass", arg("scale")="linear"));
    def("density_profile", density_profile_ptr2,(arg("x"), arg("y"), arg("z"), arg("mass"),arg("filter"), arg("nbins"), arg("rmin"), arg("rmax"), arg("filter_range"), arg("compute")="mass", arg("scale")="linear"));
    def("disp_profile", disp_profile_ptr1,(arg("x"), arg("y"), arg("z"), arg("A"), arg("nbins"), arg("rmin"), arg("rmax"), arg("scale")="linear"));
    def("disp_profile", disp_profile_ptr2,(arg("x"), arg("y"), arg("z"), arg("A"), arg("filter"), arg("nbins"), arg("rmin"), arg("rmax"), arg("filter_range"), arg("scale")="linear"));
    def("lag_rad", lag_rad_ptr1);
    def("lag_rad", lag_rad_ptr2);
    def("jacobi", &jacobi);
    def("shape", &shape);
    def("shape_profile", shape_profile_ptr1, (arg("x"), arg("y"), arg("z"), arg("mass"), arg("nbins"), arg("rmin"), arg("rmax"), arg("scale")="linear"));
    def("shape_profile", shape_profile_ptr2, (arg("x"), arg("y"), arg("z"), arg("mass"), arg("filter"), arg("nbins"), arg("rmin"), arg("rmax"), arg("filter_range"), arg("scale")="linear"));
    /* End Stellar Dynamics package*/
}    
