/***
 * File: basin_helpers.h
 * 
 * Purpose: The functions defined here are written with the intention of creating greater
 * support for some of the common interfaces and features of the Python language.
 * In general, the code here helps transform data from types specific to C++ to types that
 * are acceptable to Python, namely the boost::python::object.
 */

#ifndef __BASIN_HELPERS_H__
#define __BASIN_HELPERS_H__

#include <boost/python.hpp>
#include <boost/variant.hpp>
#include <Python.h>

/*
 * Implements a visitor that will convert a boost variant 
 * to a boost::python::object. The actual type of the data
 * in the variant doesn't need to be known, and shouldn't
 * be an issue as long as boost::python::object is capable of
 * transforming it to a python representation.
 */
struct get_py_object
    : public boost::static_visitor<boost::python::object>
{
    // TODO Add code to check that creating the boost::python::object will
    // not cause any errors 
    template<typename T>
    boost::python::object operator()(T& bv) const {
        return boost::python::object(bv);
    }
};


int get_py_object_len(boost::python::object o);


#endif //__BASIN_HELPERS_H__
