/***
 * File: basin_wrapper.cpp
 *
 * Purpose: Defines the Boost Python modules that wraps the 
 * C++ BASIN library.
 */


#include "basin_wrapper.h"

//#include <cosmology.h>
//#include <simple_plot.h>
//#include <visit_plot.h>
//#include <statistics.h>
//#include <stellardynamics.h>
//#include <transforms.h>

#include <iostream>


// wrapper defs
#include "basin_funcs_wrapper.h"
#include "Region_wrapper.h"
#include "Data_wrapper.h"
#include "List_wrapper.h"
#include "Grid_wrapper.h"
#include "Context_wrapper.h"
#include "Attribute_wrapper.h"
#include "Geometry_wrapper.h"
#include "Cosmology_wrapper.h"
#include "StellarDynamics_wrapper.h"
#include "Transforms_wrapper.h"
#include "Statistics_wrapper.h"
#include "LinearAlgebra_wrapper.h"
#include "Function_wrapper.h"
#include "Cosmosims_wrapper.h"

#ifdef VISIT_SUPPORT
#include "Visit_module_wrapper.h"
#endif

using namespace std;


//void basinInit() {
//    int myid, numprocs;
//
//    //basin_initialize(myid, numprocs);
//    //Testing here for running MPI in multi-threaded mode
//    int ret = MPI::Init_thread(MPI_THREAD_MULTIPLE);
//    if(ret == MPI_THREAD_MULTIPLE)
//        cerr << "Basin initialized with full threads\n";
//    else
//        cerr << "THREADING NOT SUPPORTED, INITIALIZED!\n";
//
//    MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
//    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
//    BASIN_NPROCS=numprocs;
//    BASIN_ID=myid;
//}

void basinInit() {
    int myid, numprocs;
    basin_initialize(myid, numprocs);
}

//A simple wrapper for MPI_Barrier() so we can 
//use the barrier in Python
void basin_mpi_barrier() {
    MPI_Barrier(MPI_COMM_WORLD);
}

int get_basin_id() {
    return BASIN_ID;
}

int get_basin_nprocs() {
    return BASIN_NPROCS;
}

void printHello() {
    cout << "Hello! from B_ID " << BASIN_ID << endl;
    cout << "World size is " << BASIN_NPROCS << endl;
}



// new module called "_basin" will start here
// The actual "basin" module is created in basin.py
// The user should only ever import "basin" and not
// "_basin"
BOOST_PYTHON_MODULE(_basin) {
    //make sure MPI environment is setup before anything else happens
    basinInit();

    // wrapper definition from basin_funcs_wrapper.cpp
    export_basin_funcs();

    // wrapper definition from Region_wrapper.cpp
    export_Region();

    // wrapper definition from Data_wrapper.cpp
    export_Data();

    // wrapper definition from List_wrapper.cpp
    export_List();

    // wrapper definition from Grid_wrapper.cpp
    export_Grid();
    
    // wrapper definition from Context_wrapper.cpp
    export_Context();

    // wrapper definition from Attribute_wrapper.cpp
    export_Attribute();

    // wrapper definition from Geometry_wrapper.cpp
    export_Geometry();

    // wrapper definition from Cosmology_wrapper.cpp
    export_Cosmology();

    // wrapper definition from StellarDynamics_wrapper.cpp
    export_StellarDynamics();

    // wrapper definitions from Transforms_wrapper.cpp
    export_Transforms();

    // wrapper definitions from Statistics_wrapper.cpp
    export_Statistics();

    // wrapper definitions from Statistics_wrapper.cpp
    export_Function();

    // wrapper definitions from Cosmosims_wrapper.cpp
    export_Cosmosims();

    // wrapper definitions from Visit_module_wrapper.cpp
#ifdef VISIT_SUPPORT
    export_Visit_module();
#endif


    /* Some helper functions */ 
    def("print_hello", &printHello);
    def("basin_mpi_barrier", &basin_mpi_barrier);
    def("get_basin_id", &get_basin_id);
    def("get_basin_nprocs", &get_basin_nprocs);
  
    /* Add support for wrapping vectors and other containers*/
    container_from_list<vector<int> >();
    container_from_list<vector<double> >();
    container_from_list<vector<float> >();
    to_python_converter<vector<int>, container_to_list<vector<int> > >();
    to_python_converter<vector<double>, container_to_list<vector<double> > >();
    to_python_converter<vector<float>, container_to_list<vector<float> > >();
    to_python_converter<vector<vector<double> >, container_to_list<vector<vector<double> > > >();
    to_python_converter<vector<string>, container_to_list<vector<string> > >();     
    to_python_converter<vector<bv1>, variant_vector_to_list >();

    /* Convert BASIN matrix to Python numpy matrix
     * Conversion code found in LinearAlgebra_wrapper.h
     */
    to_python_converter<basin_matrix, basin_matrix_to_py_matrix>();

}



